#include <QString>
#include <string>
#include <QFile>
#include <QtGui>

#include "mainwindow.hpp"

#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/tree_iarchive.hpp>
#include <boost/archive/tree_oarchive.hpp>

#include <boost/filesystem/convenience.hpp>

MainWindow::MainWindow() : QMainWindow(), m_tree(0)
{
	m_file_menu = menuBar()->addMenu(tr("&File"));
	m_file_menu->addAction(tr("&Open..."), this, SLOT(openFile()), QKeySequence(tr("Ctrl+O")));
	m_file_menu->addAction(tr("&Save..."), this, SLOT(saveFile()), QKeySequence(tr("Ctrl+S")));
	m_file_menu->addAction(tr("E&xit"), this, SLOT(close()), QKeySequence(tr("Ctrl+Q")));

	m_tree = new qt_tree<char>(this);
	m_view = new QTreeView(this);
	m_view->setModel(m_tree);

	setCentralWidget(m_view);
	setWindowTitle(tr("GUI Tree, boost::serialization test"));
};

void MainWindow::openFile()
{
	std::string file_name = QFileDialog::getOpenFileName(this, tr("Open File"), ".",
			tr("XML boost::serialization files (*.xml);;"
			   "TXT boost::serialization files (*.txt)")).toStdString();

	if(file_name.empty())
		return;

	std::ifstream is(file_name.c_str());
	assert(is.good());
	if(boost::filesystem::extension(file_name)==".txt")
	{
		boost::archive::text_iarchive ia(is);
		ia >> boost::serialization::make_nvp("Bus_Schedules", m_bs);
	}
	else 
	if(boost::filesystem::extension(file_name)==".xml")
	{
		boost::archive::xml_iarchive ia(is);
		ia >> boost::serialization::make_nvp("Bus_Schedules", m_bs);
	}

	{
		boost::archive::tree_oarchive oa(*m_tree);
		oa << boost::serialization::make_nvp("Bus_Schedules", m_bs);
	}
	m_view->setModel(m_tree);
	m_view->setUniformRowHeights(true);
	//m_view->expandAll(); // qt4.2
	expandAll(m_view);
	m_view->resizeColumnToContents(0);
	m_view->resizeColumnToContents(1);
	//m_view->expand(m_tree->index(0,0));

//	m_tree->doExpand(0,0,2,m_view);
//		void qt_tree::doExpand(int row, int column, int levels, QTreeView* tv)
//		{
//			qt_tree_node<Elem> *parentItem = ((qt_tree_node<Elem>*)tree<Elem>::m_ctrl);
//			qt_tree_node<Elem> *childItem = parentItem->child(row);
//			while( childItem && levels--)
//			{
//				tv->expand(index(row,column,createIndex(row,column,parentItem)));
//				parentItem = childItem;
//				childItem = parentItem->child(row);
//			}
//		};
	
//	std::ofstream os("demofile.xml");
//	{
//		boost::archive::xml_oarchive oa(os);
//		oa << boost::serialization::make_nvp("Bus Schedules", m_bs);
//	}
//	{
//		std::ofstream ofs("demofile.xml");
//		assert(ofs.good());
//		boost::archive::xml_oarchive oa(ofs);
////		oa << BOOST_SERIALIZATION_NVP(m_bs);
//		oa << boost::serialization::make_nvp("Bus_Schedules", m_bs);
//	}


};


void MainWindow::saveFile()
{
	std::string file_name = QFileDialog::getSaveFileName(this, tr("Save File"), ".",
			tr("XML boost::serialization files (*.xml);;"
			   "TXT boost::serialization files (*.txt)")).toStdString();

	if(file_name.empty())
		return;

	{
		boost::archive::tree_iarchive ia(*m_tree);
		ia >> boost::serialization::make_nvp("Bus_Schedules", m_bs);
	}
	if(boost::filesystem::extension(file_name)=="")
		file_name+=".xml";
	std::ofstream os(file_name.c_str());
	assert(os.good());
	if(boost::filesystem::extension(file_name)==".txt")
	{
		boost::archive::text_oarchive oa(os);
		oa << boost::serialization::make_nvp("Bus_Schedules", m_bs);
	}
	else
	{
		boost::archive::xml_oarchive oa(os);
		oa << boost::serialization::make_nvp("Bus_Schedules", m_bs);
	}
};


void MainWindow::expandAll(QTreeView* view, const QModelIndex& parent)
{
	for (int i = 0; i < view->model()->rowCount(parent); ++i) 
	{
		QModelIndex index = view->model()->index(i, 0, parent);
		view->expand(index);
//		const char * buf = ((tree_node<char>*)index.internalPointer())->m_item_name;
//		std::cout << i << " m_item_name " << (buf?buf:"") << "\n";
		expandAll(view, index);
	}
}

